import os

import audiostack
from audiostack.content.script import Script

audiostack.api_base = os.environ.get("AUDIO_STACK_DEV_URL", "https://v2.api.audio")
audiostack.api_key = os.environ["AUDIO_STACK_DEV_KEY"]  # type: ignore

SCRIPT_ID = ""


def test_create() -> None:
    script_item = Script.create(
        scriptText="hello sam",
        projectName="sdk_test",
        moduleName="sdk_test",
        scriptName="sdk_1",
    )
    global SCRIPT_ID
    SCRIPT_ID = script_item.scriptId


def test_get() -> None:
    script_item = Script.get(scriptId=SCRIPT_ID)
    assert script_item


def test_update() -> None:
    Script.update(scriptId=SCRIPT_ID, scriptText="blah blah")


def test_preview() -> None:
    script_item = Script.get(scriptId=SCRIPT_ID, previewWithVoice="sara")
    assert script_item


def test_delete() -> None:
    Script.delete(scriptId=SCRIPT_ID)


def test_inbuilts() -> None:
    script_item = Script.create(
        scriptText="hello sam",
        projectName="sdk_test",
        moduleName="sdk_test",
        scriptName="sdk_1",
    )
    script_item.update(scriptText="this is an update")
    script_item.delete()


def test_list() -> None:
    items = Script.list()
    for i in items:
        assert isinstance(i, Script.Item)
        assert i.scriptId



import os

import audiostack
from audiostack.content.file import File

audiostack.api_base = os.environ.get("AUDIO_STACK_DEV_URL", "https://v2.api.audio")
audiostack.api_key = os.environ["AUDIO_STACK_DEV_KEY"]  # type: ignore

test_constants = {}


def test_create() -> None:
    r = File.create(localPath="example.mp3", uploadPath="example.mp3", fileType="audio")
    test_constants["fileId"] = r.fileId
    print(r)


def test_get() -> None:
    r = File.get(test_constants["fileId"])
    print(r)


def test_modify() -> None:
    r = File.modify(fileId=test_constants["fileId"], category="test")
    print(r)


def test_search() -> None:
    files = File.search()
    for f in files:
        print(f)

    files = File.search(source="pythonSDK")
    for f in files:
        print(f)


def test_delete() -> None:
    r = File.get(test_constants["fileId"])
    r2 = r.delete()
    print(r2)


def test_create_2() -> None:
    r = File.create(
        localPath="example.mp3",
        uploadPath="example.mp3",
        fileType="audio",
        category="sounds",
        tags=["a", "b"],
        metadata={"hello": "world"},
    )
    test_constants["fileId"] = r.fileId
    print(r)
    r.delete()



import os

import audiostack
from audiostack.content.file import File

audiostack.api_base = os.environ.get("AUDIO_STACK_DEV_URL", "https://v2.api.audio")
audiostack.api_key = os.environ["AUDIO_STACK_DEV_KEY"]  # type: ignore

test_constants = {}  # type: dict


def test_transfer() -> None:
    script = audiostack.Content.Script.create(scriptText="hello sam")

    speech = audiostack.Speech.TTS.create(scriptItem=script, voice="sara")

    mix = audiostack.Production.Mix.create(speechItem=speech)
    print(mix)

    r = File.transfer(
        url=mix.data["files"][0]["url"], uploadPath="mymix.wav", tags=["a"]
    )
    print(r)

    delivery = audiostack.Delivery.Encoder.encode_mix(
        productionItem=mix, preset="mp3_very_high"
    )
    print(delivery, delivery.url)

    r2 = File.transfer(url=delivery.url, uploadPath="mymix.mp3", tags=["a"])
    print(r2)

    r.delete()
    r2.delete()



import os

import audiostack
from audiostack.content.file import File

audiostack.api_base = os.environ.get("AUDIO_STACK_DEV_URL", "https://v2.api.audio")
audiostack.api_key = os.environ["AUDIO_STACK_DEV_KEY"]  # type: ignore


def test_create() -> None:
    r = File.create(localPath="example.mp3", uploadPath="example.mp3", fileType="audio")
    print("fileId: ", r.fileId)
    aesItem = audiostack.Production.Suite.evaluate(fileId=r.fileId)
    assert aesItem.status_code == 200, "BiG ERROR"



from audiostack.speech.diction import Diction  # noqa: F401
from audiostack.speech.predict import Predict  # noqa: F401
from audiostack.speech.tts import TTS  # noqa: F401
from audiostack.speech.voice import Voice  # noqa: F401



import os

import audiostack
from audiostack.speech.diction import Diction

audiostack.api_base = os.environ.get("AUDIO_STACK_DEV_URL", "https://v2.api.audio")
audiostack.api_key = os.environ["AUDIO_STACK_DEV_KEY"]  # type: ignore


def test_dict_list() -> None:
    # list words in default public dicts
    dicts = Diction.list()
    for d in dicts:
        assert d
        print(d.lang, d.content, d.words[0:10])


def test_dict_custom_list() -> None:
    # list words in custom dicts
    dicts = Diction.Custom.list()
    for d in dicts:
        print(d.lang, d.content, d.words[0:10])


def test_create() -> None:
    create = Diction.Custom.create_word(
        word="AFLR", replacement="aflorithmic", lang="global"
    )
    assert create


def test_delete() -> None:
    response = Diction.Custom.delete_word(word="AFLR", lang="global")
    print(response)


def test_list_custom() -> None:
    # list words and replacements in a custom dict
    word_sets = Diction.Custom.list_words(lang="global")
    for w in word_sets:
        print(w.inputs, w.replacements)



import os

import audiostack
from audiostack.speech.predict import Predict

audiostack.api_base = os.environ.get("AUDIO_STACK_DEV_URL", "https://v2.api.audio")
audiostack.api_key = os.environ["AUDIO_STACK_DEV_KEY"]  # type: ignore

long_text = """
In the sprawling city of Techville, nestled amidst a forest of towering skyscrapers, there lived a software developer named lars. He was an introverted genius, known throughout the tech industry for his unparalleled coding skills. But there was something peculiar about lars that sent shivers down the spines of his coworkers—he had an obsession with the darker side of programming.
lars's fascination with the arcane and forbidden had led him to a remote corner of the internet where he stumbled upon a mysterious chat room. It was there that he encountered a shadowy group of hackers who called themselves The Black Coders. They promised power, wealth, and knowledge beyond imagination to anyone who could complete their sinister tasks.
Intrigued and intoxicated by the allure of forbidden knowledge, lars agreed to join their ranks. Little did he know that he had just crossed a line from which there was no return.
"""


def test_list() -> None:
    voices = Predict.list()
    print(voices)
    for v in voices:
        assert isinstance(v, str)


def test_predict() -> None:
    item = Predict.predict(text=long_text, voice="sara")
    assert item.length  # type: ignore



import json
import shutil
from typing import Any, Callable, Dict, Optional, Union

import requests

import audiostack
from audiostack.helpers.request_types import RequestTypes


def remove_empty(data: Any) -> Any:
    if not (isinstance(data, dict) or isinstance(data, list)):
        return data

    final_dict = {}
    for key, val in data.items():  # type: ignore
        if val or isinstance(val, int):  # val = int(0) shoud not be removed
            if isinstance(val, dict):
                final_dict[key] = remove_empty(val)
            elif isinstance(val, list):
                final_dict[key] = list(filter(None, [remove_empty(i) for i in val]))
            else:
                final_dict[key] = val
    return final_dict


class RequestInterface:
    # disable debug print
    DEBUG_PRINT = False

    def __init__(self, family: str) -> None:
        self.family = family

    @staticmethod
    def make_header() -> dict:
        header = {
            "x-api-key": audiostack.api_key,
            "x-python-sdk-version": audiostack.sdk_version,
        }
        if audiostack.assume_org_id:
            header["x-assume-org"] = audiostack.assume_org_id
        return header

    def resolve_response(self, r: Any) -> dict:
        if self.DEBUG_PRINT:
            print(json.dumps(r.json(), indent=4))
        if r.status_code >= 500:
            raise Exception("Internal server error - aborting")

        if r.status_code == 403:
            exc = r.json().get("message", "Not authorised - check API key is valid")
            raise Exception(exc)

        if r.status_code >= 400:
            msg = (
                r.json()["message"]
                + ". Errors listed as follows: \n\t"
                + "\t".join(r.json()["errors"])
            )
            raise Exception(msg)

        if "meta" in r.json():
            if "creditsUsed" in r.json()["meta"]:
                audiostack.billing_session += r.json()["meta"]["creditsUsed"]

        return {**r.json(), **{"statusCode": r.status_code}}

    def send_upload_request(self, local_path: str, upload_url: str) -> int:
        with open(local_path, "rb") as data:
            r = requests.put(url=upload_url, data=data)

            if r.status_code >= 400:
                raise Exception("Failed to upload file")

            return r.status_code

    def send_request(
        self,
        rtype: str,
        route: str,
        json: Optional[dict] = None,
        path_parameters: Optional[Union[dict, str]] = None,
        query_parameters: Optional[Union[dict, str]] = None,
        overwrite_base_url: Optional[str] = None,
    ) -> Any:
        if overwrite_base_url:
            url = overwrite_base_url
        else:
            if self.family:
                url = f"{audiostack.api_base}/{self.family}"
            else:
                url = audiostack.api_base

        if route:
            url += "/" + route

        if rtype not in RequestTypes.valid_types:
            assert False

        if json:
            json = remove_empty(json)
        if query_parameters:
            query_parameters = remove_empty(query_parameters)

        # these requests are all the same input parameters.
        if rtype in [RequestTypes.POST, RequestTypes.PUT, RequestTypes.PATCH]:
            FUNC_MAP: Dict[str, Callable] = {
                RequestTypes.POST: requests.post,
                RequestTypes.PUT: requests.put,
                RequestTypes.PATCH: requests.patch,
            }

            return self.resolve_response(
                FUNC_MAP[rtype](url=url, json=json, headers=self.make_header())
            )
        elif rtype == RequestTypes.GET:
            if path_parameters:
                url = f"{url}/{path_parameters}"

            return self.resolve_response(
                requests.get(
                    url=url, params=query_parameters, headers=self.make_header()
                )
            )
        elif rtype == RequestTypes.DELETE:
            if path_parameters:
                url = f"{url}/{path_parameters}"

            return self.resolve_response(
                requests.delete(
                    url=url, params=query_parameters, headers=self.make_header()
                )
            )

    @classmethod
    def download_url(cls, url: str, name: str, destination: str) -> None:
        r = requests.get(url=url, stream=True, headers=cls.make_header())

        if r.status_code >= 400:
            raise Exception("Failed to download file")

        local_filename = f"{destination}/{name}"
        with open(local_filename, "wb") as f:
            shutil.copyfileobj(r.raw, f)



from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes


class Documentation:
    interface = RequestInterface(family="")

    @staticmethod
    def docs_for_service(service: object) -> APIResponseItem:
        service = service.__name__.lower()  # type: ignore

        r = Documentation.interface.send_request(
            rtype=RequestTypes.GET,
            route="documentation",
            query_parameters={"route": service},
        )
        return APIResponseItem(r)



from typing import Any, Optional

from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes
from audiostack.production.suite import Suite


class Video:
    interface = RequestInterface(family="delivery")

    class Item(APIResponseItem):
        def __init__(self, response: dict) -> None:
            super().__init__(response)
            self.url = self.data["url"]
            self.format = self.data.get("format", "mp4")

        def download(self, fileName: str = "default", path: str = "./") -> None:
            full_name = f"{fileName}.{self.format}"
            RequestInterface.download_url(self.url, destination=path, name=full_name)

    @staticmethod
    def create_from_production_and_image(
        productionId: str = "",
        productionItem: Optional[Any] = None,
        public: bool = False,
    ) -> Item:
        if productionId and productionItem:
            raise Exception(
                "productionId or productionItem should be supplied not both"
            )
        if not (productionId or productionItem):
            raise Exception("productionId or productionItem should be supplied")

        if productionItem:
            try:
                productionId = productionItem.productionId
            except Exception:
                raise Exception(
                    "supplied productionItem is missing an attribute, productionItem should be type object and a response from Production.Mix"
                )
        elif productionId:
            if not isinstance(productionId, str):
                raise Exception("supplied productionId should be a uuid string.")
        body = {
            "productionId": productionId,
            "public": public,
        }
        r = Video.interface.send_request(
            rtype=RequestTypes.POST, route="video", json=body
        )
        retries = 0
        max_retries = 30
        while r["statusCode"] == 202 and retries < max_retries:
            print("Response in progress please wait...")
            videoId = r["data"]["videoId"]
            r = Video.interface.send_request(
                rtype=RequestTypes.GET, route="video", path_parameters=videoId
            )
            retries += 1
        return Video.Item(r)

    @staticmethod
    def create_from_production_and_video(
        productionId: str = "",
        productionItem: Optional[Any] = None,
        videoFileId: str = "",
        mode: dict = {},
        public: bool = False,
    ) -> Item:
        if productionId and productionItem:
            raise Exception(
                "productionId or productionItem should be supplied not both"
            )
        if not (productionId or productionItem):
            raise Exception("productionId or productionItem should be supplied")

        if productionItem:
            try:
                productionId = productionItem.productionId
            except Exception:
                raise Exception(
                    "supplied productionItem is missing an attribute, productionItem should be type object and a response from Production.Mix"
                )
        elif productionId:
            if not isinstance(productionId, str):
                raise Exception("supplied productionId should be a uuid string.")
        body = {
            "productionId": productionId,
            "public": public,
            "videoFileId": videoFileId,
            "mode": mode,
            "format": "",
        }
        r = Video.interface.send_request(
            rtype=RequestTypes.POST, route="video", json=body
        )
        retries = 0
        max_retries = 30
        while r["statusCode"] == 202 and retries < max_retries:
            print("Response in progress please wait...")
            videoId = r["data"]["videoId"]
            r = Video.interface.send_request(
                rtype=RequestTypes.GET, route="video", path_parameters=videoId
            )
            retries += 1
        return Video.Item(r)

    @staticmethod
    def create_from_file_and_video(
        fileId: str = "",
        videoFileId: str = "",
        mode: dict = {},
    ) -> Item:
        interface = RequestInterface(family="production")

        if fileId and not videoFileId:
            raise Exception(
                "if videoFileId is supplied, fileId should be supplied as well"
            )

        body = {
            "fileId": fileId,
            "videoFileId": videoFileId,
            "public": False,
            "outputFormat": "",
            "mode": mode,
        }

        r = interface.send_request(
            rtype=RequestTypes.POST, route="suite/file_to_video", json=body
        )

        item = Suite.PipelineInProgressItem(r)
        return Video.Item(_poll_video(r, item.pipelineId))

    @staticmethod
    def create_from_file_and_image(fileId: str = "") -> Item:
        interface = RequestInterface(family="production")

        body = {
            "fileId": fileId,
            "public": False,
            "outputFormat": "mp4",
            "mode": {"setting": "default"},
        }

        r = interface.send_request(
            rtype=RequestTypes.POST, route="suite/file_to_video", json=body
        )

        item = Suite.PipelineInProgressItem(r)
        return Video.Item(_poll_video(r, item.pipelineId))


def _poll_video(r: dict, pipelineId: str) -> dict:
    while r["statusCode"] == 202:
        interface = RequestInterface(family="production")
        print("Response in progress please wait...")
        r = interface.send_request(
            rtype=RequestTypes.GET,
            route="suite/videopipeline",
            path_parameters=pipelineId,
        )
    status = r.get("data", {}).get("status", 200)
    if status > 400:
        msg = r.get("data", {}).get("message")
        errors = r.get("data", {}).get("errors")
        raise Suite.FailedPipeline(
            "pipeline failed: ", msg, "errors are as follows: ", ",".join(errors)
        )

    return r



from typing import Any

from .api_item import APIResponseItem


class APIResponseList(APIResponseItem):
    def __init__(self, response: dict, list_type: str) -> None:
        super().__init__(response)
        self.idx = 0
        self.list_type = list_type
        self.items = self.response["data"][list_type]

    def __iter__(self) -> "APIResponseList":
        return self

    def __next__(self) -> Any:
        self.idx += 1
        try:
            item = self.items[self.idx - 1]
            return self.resolve_item(self.list_type, item)

        except IndexError:
            self.idx = 0
            raise StopIteration

    def __getitem__(self, x: Any) -> Any:
        if isinstance(x, slice):
            return [
                self.resolve_item(self.list_type, x)
                for x in self.items[x.start : x.stop]
            ]
        else:
            return self.resolve_item(self.list_type, x)

    # child classes should override this method, failing to do so will raise an exception!
    def resolve_item(self, list_type: str, item: Any) -> Any:
        raise Exception()



import os
from typing import Any

from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList
from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes


class Media:
    FAMILY = "content"
    interface = RequestInterface(family=FAMILY)

    class Item(APIResponseItem):
        def __init__(self, response: dict) -> None:
            super().__init__(response)

            self.mediaId = self.data["mediaId"]
            self.tags = self.data["tags"]
            self.filename = self.data["filename"]

        def delete(self) -> APIResponseItem:
            return Media.delete(self.mediaId)

    class List(APIResponseList):
        def __init__(self, response: dict, list_type: str) -> None:
            super().__init__(response, list_type)

        def resolve_item(self, list_type: str, item: Any) -> "Media.Item":
            if list_type == "mediaFiles":
                return Media.Item({"data": item})
            else:
                raise Exception()

    @staticmethod
    def create(filePath: str) -> Item:
        if not os.path.isfile(filePath):
            raise Exception("Supplied file does not exist")

        name = filePath.rpartition("/")[2]

        r = Media.interface.send_request(
            rtype=RequestTypes.POST,
            route="media/create-upload-url",
            json={"fileName": name},
        )
        response = APIResponseItem(r)
        url = response.data["fileUploadUrl"]
        mediaId = response.data["mediaId"]

        Media.interface.send_upload_request(local_path=filePath, upload_url=url)
        return Media.get(mediaId)

    @staticmethod
    def get(mediaId: str) -> Item:
        r = Media.interface.send_request(
            rtype=RequestTypes.GET, route="media", path_parameters=mediaId
        )
        return Media.Item(r)

    @staticmethod
    def delete(mediaId: str) -> APIResponseItem:
        r = Media.interface.send_request(
            rtype=RequestTypes.DELETE, route="media", path_parameters=mediaId
        )
        return APIResponseItem(r)

    @staticmethod
    def list() -> List:
        r = Media.interface.send_request(rtype=RequestTypes.GET, route="media")
        return Media.List(r, list_type="mediaFiles")



from typing import Any

from audiostack.helpers.api_item import APIResponseItem
from audiostack.helpers.api_list import APIResponseList
from audiostack.helpers.request_interface import RequestInterface
from audiostack.helpers.request_types import RequestTypes


class Voice:
    interface = RequestInterface(family="speech/voice")

    class Item(APIResponseItem):
        def __init__(self, response: dict) -> None:
            super().__init__(response)
            self.provider = self.data["provider"]
            self.alias = self.data["alias"]

    class List(APIResponseList):
        def __init__(self, response: dict, list_type: str) -> None:
            super().__init__(response, list_type)

        def resolve_item(self, list_type: str, item: Any) -> "Voice.Item":
            if list_type == "voices":
                return Voice.Item({"data": item})
            else:
                raise Exception()

    @staticmethod
    def select_for_script(
        scriptId: str = "", scriptItem: Any = "", tone: str = "", targetLength: int = 20
    ) -> APIResponseItem:
        if scriptId and scriptItem:
            raise Exception("scriptId or scriptItem should be supplied not both")
        if not (scriptId or scriptItem):
            raise Exception("scriptId or scriptItem should be supplied")

        if scriptItem:
            scriptId = scriptItem.scriptId

        body = {"scriptId": scriptId, "tone": tone, "targetLength": targetLength}

        r = Voice.interface.send_request(
            rtype=RequestTypes.POST, route="select", json=body
        )
        return APIResponseItem(r)

    @staticmethod
    def select_for_content(content: str, tone: str = "") -> APIResponseItem:
        body = {"content": content}
        if tone:
            body["tone"] = tone

        r = Voice.interface.send_request(
            rtype=RequestTypes.POST, route="select", json=body
        )
        return APIResponseItem(r)

    @staticmethod
    def list() -> "Voice.List":
        r = Voice.interface.send_request(rtype=RequestTypes.GET, route="")
        return Voice.List(r, list_type="voices")

    class Parameter:
        @staticmethod
        def get() -> APIResponseItem:
            r = Voice.interface.send_request(rtype=RequestTypes.GET, route="parameter")
            return APIResponseItem(r)
